<?php

namespace Drupal\geolocation_provider\Controller;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Controller\ControllerBase;
use Drupal\geolocation_provider\Plugin\GeolocationProviderPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\dialog\Plugin\DialogManager;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class GeolocationProviderController.
 */
class GeolocationProviderController extends ControllerBase {

  /**
   * The geolocation plugin manager.
   *
   * @var \Drupal\geolocation_provider\Plugin\GeolocationProviderPluginManager
   */
  protected $geolocationManager;

  /**
   * Constructs a new DialogController object.
   *
   * @param \Drupal\geolocation_provider\Plugin\GeolocationProviderPluginManager $geolocation_manager
   *   The geolocation manager.
   */
  public function __construct(GeolocationProviderPluginManager $geolocation_manager) {
    $this->geolocationManager = $geolocation_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.geolocation_provider_plugin')
    );
  }

  /**
   * Geolocation callback.
   *
   * @param string $plugin
   *   A plugin id.
   * @param string $search
   *   A string representing the address to geoloc.
   *
   * @return JsonResponse
   *   The json response send to the user.
   */
  public function geolocationCallback($plugin, $search) {
    /** @var \Drupal\geolocation_provider\Plugin\GeolocationProviderPluginInterface $instance */
    $instance = NULL;
    $error = '';
    try {
      $instance = $this->geolocationManager->createInstance($plugin);
    }
    catch (PluginException $e) {
      $error = $e->getMessage();
      \Drupal::logger('geolocation_provider')
        ->error('GeolocationProviderController::geolocationCallback error: @error', [
          '@error' => $error,
        ]);
    }
    if (!empty($instance)) {
      return new JsonResponse($instance->geolocation($search, [], TRUE));
    }
    return new JsonResponse($error, 500);
  }

  /**
   * Reverse callback.
   *
   * @param string $plugin
   *   A plugin id.
   * @param float $lat
   *   The latitude used for the reverse search.
   * @param float $lon
   *   The longitude used for the reverse search.
   *
   * @return JsonResponse
   *   The json response send to the user.
   */
  public function reverseCallback($plugin, $lat, $lon) {
    /** @var \Drupal\geolocation_provider\Plugin\GeolocationProviderPluginInterface $instance */
    $instance = NULL;
    $error = '';
    try {
      $instance = $this->geolocationManager->createInstance($plugin);
    }
    catch (PluginException $e) {
      $error = $e->getMessage();
      \Drupal::logger('geolocation_provider')
        ->error('GeolocationProviderController::geolocationCallback error: @error', [
          '@error' => $error,
        ]);
    }
    if (!empty($instance)) {
      return new JsonResponse((array) $instance->reverse($lat, $lon, TRUE));
    }
    return new JsonResponse($error, 500);
  }

}
